package at.kugel.tool.buildtray;

import java.awt.AWTException;
import java.awt.Image;
import java.awt.MenuItem;
import java.awt.PopupMenu;
import java.awt.SystemTray;
import java.awt.TrayIcon;
import java.awt.TrayIcon.MessageType;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.io.PrintWriter;
import java.io.StringWriter;
import java.util.EnumMap;

import at.kugel.tool.buildtray.status.Status;
import at.kugel.tool.buildtray.status.SetStatusAble;
import at.kugel.tool.buildtray.action.Actions;

/**
 * Setup the tray with popup menu and image icon list. Provides methods to change the tray icon and popup messages. This
 * just wrapps around the tray icon.
 */
public class Tray implements SetStatusAble {

   private final EnumMap<Status, Image> images;
   private final TrayIcon trayIcon;

   private Status status = Status.BUSY;
   private String projectName;
   private final String initialTip = "Build Status Tray";

   // --- setup ---

   public Tray(EnumMap<Status, Image> pImages, final Actions pActions) throws AWTException {
      images = pImages;

      // load an image
      Image image = images.get(status);

      // construct a TrayIcon
      trayIcon = new TrayIcon(image, initialTip, createPopup(pActions));

      // create a action listener to listen for default action executed on the tray icon
      final ActionListener defaultListener = new ActionListener() {
         public void actionPerformed(ActionEvent e) {
            // execute default action of the application
            pActions.openBrowser();
         }
      };
      // set the TrayIcon properties
      trayIcon.addActionListener(defaultListener);

      // get the SystemTray instance
      final SystemTray tray = SystemTray.getSystemTray();
      // add the tray image
      tray.add(trayIcon);
   }

   private PopupMenu createPopup(final Actions pActions) {
      // create a popup menu
      final PopupMenu popup = new PopupMenu();
      // create menu item for the default action
      MenuItem item = new MenuItem("open");
      item.addActionListener(new ActionListener() {
         public void actionPerformed(ActionEvent e) {
            // execute default action of the application
            pActions.openBrowser();
         }
      });
      popup.add(item);

      item = new MenuItem("refresh");
      item.addActionListener(new ActionListener() {
         public void actionPerformed(ActionEvent e) {
            // execute default action of the application
            pActions.refresh(Tray.this);
         }
      });
      popup.add(item);

      popup.addSeparator();

      item = new MenuItem("exit");
      item.addActionListener(new ActionListener() {
         public void actionPerformed(ActionEvent pE) {
            pActions.exit();
         }
      });
      popup.add(item);

      return popup;
   }

   // --- change icon ---

   public void setStatus(Status pStatus, String pProjectName, String pDate) {
      // some time later the application state has changed - update the image
      final Status lastStatus = status;
      status = pStatus;
      trayIcon.setImage(images.get(status));
      final String lastProjectName = projectName;
      projectName = pProjectName;
      switch (pStatus) {
         case OK:
            trayIcon.setToolTip("all are " + pStatus + ' ' + pDate);
            break;
         case FAILED:
            trayIcon.setToolTip(pProjectName + " is " + pStatus + ' ' + pDate);
            if (status.ordinal() > lastStatus.ordinal() && projectName.equals(lastProjectName)) {
               // changed from busy to failed on same project, make a warning
               displayMessage("Build Failed", pProjectName + " is " + pStatus + ' ' + pDate, TrayIcon.MessageType.NONE);
            }
            break;
         case BUSY:
            trayIcon.setToolTip(pProjectName + " is " + pStatus);
            if (lastProjectName != null && !projectName.equals(lastProjectName)) {
               // we are busy now, maybe in new project
               displayMessage("New Build", "Finished " + lastProjectName + "\nBuilding " + projectName, TrayIcon.MessageType.NONE);
            }
            break;
         default:
            throw new IllegalArgumentException("unknown value " + pStatus);
      }
   }

   public void setError(Exception pException) {
      final StringWriter out = new StringWriter();
      pException.printStackTrace(new PrintWriter(out));
      displayMessage("Error " + pException.toString(), out.toString(), TrayIcon.MessageType.ERROR);
   }

   /**
    * Wrapper for the tray icon's popup, which does not work on windows XP (known bug).
    * @see TrayIcon#displayMessage
    */
   private void displayMessage(String pCaption, String pText, MessageType pMessageType) {
      if ("5.1".equals(System.getProperty("os.version"))) {
         new TrayPopup().displayMessage(pCaption, pText);
      } else {
         trayIcon.displayMessage(pCaption, pText, pMessageType);
      }
   }
}