package at.kugel.tool.buildtray.action;

import java.awt.Desktop;
import java.io.IOException;
import java.net.URI;
import java.net.URISyntaxException;
import java.net.URL;
import java.text.ParseException;
import java.text.SimpleDateFormat;
import java.util.Date;
import java.util.regex.Matcher;
import java.util.regex.Pattern;

import at.kugel.tool.buildtray.config.Config;
import at.kugel.tool.buildtray.status.SetStatusAble;
import at.kugel.tool.buildtray.status.Status;

/**
 * Implementation of tray actions using regular expressions from the config.
 */
public class HtmlRegularExpressionActions implements Actions {

   private final Config config;
   private final SimpleDateFormat dateFormat;
   private final Matcher busyMatcher;
   private final Matcher statusMatcher;

   public HtmlRegularExpressionActions(Config pConfig) {
      config = pConfig;
      busyMatcher = Pattern.compile(config.getBusyPattern()).matcher("");
      statusMatcher = Pattern.compile(config.getStatusPattern(), Pattern.DOTALL).matcher("");
      dateFormat = new SimpleDateFormat(config.getDatePattern());
   }

   public void openBrowser() {
      if (Desktop.isDesktopSupported()) {
         final Desktop dt = Desktop.getDesktop();
         if (dt.isSupported(Desktop.Action.BROWSE)) {
            try {
               dt.browse(new URI(config.getServerlUrl()));
            } catch (final IOException e) {
               // config error
               handleError(null, e);
            } catch (final URISyntaxException e) {
               // config error
               handleError(null, e);
            }
         }
      }
   }

   public void refresh(SetStatusAble pTray) {
      if (pTray == null) {
         return;
      }

      try {
         final String content = new StringIOHelper().readNet(new URL(config.getServerlUrl()));

         if (config.isCheckBusy()) {
            busyMatcher.reset(content);
            if (busyMatcher.find()) {
               final String name = busyMatcher.group(config.getBusyPatternNameGroup());
               pTray.setStatus(Status.BUSY, name, null);
               return;
            }
         }

         parseRegular(content, pTray);
      } catch (final IOException e) {
         handleError(pTray, e);
      } catch (final ParseException e) {
         handleError(pTray, e);
      }
   }

   private void parseRegular(String pContent, SetStatusAble pTray) throws ParseException {
      String lastName = null;
      Status lastStatus = Status.OK;
      Date lastDate = new Date(0);

      statusMatcher.reset(pContent);
      while (statusMatcher.find()) {
         final String name = statusMatcher.group(config.getStatusPatternNameGroup());

         final String statusStr = statusMatcher.group(config.getStatusPatternStatusGroup());
         final Status status;
         if (statusStr.matches(config.getSuccessValue())) {
            status = Status.OK;
         } else if (statusStr.matches(config.getFailedValue())) {
            status = Status.FAILED;
         } else {
            status = Status.UNKNOWN;
            continue;
         }

         final Date date = dateFormat.parse(statusMatcher.group(config.getStatusPatternDateGroup()));

         if (date.after(lastDate) && status.ordinal() >= lastStatus.ordinal()) {
            // System.out.println(name + " after " + lastName);
            lastName = name;
            lastStatus = status;
            lastDate = date;
         } else if (status.ordinal() > lastStatus.ordinal()) {
            // System.out.println(name + " worse than " + lastName);
            lastName = name;
            lastStatus = status;
            lastDate = date;
         }

      }
      pTray.setStatus(lastStatus, lastName, dateFormat.format(lastDate));
   }

   private void handleError(SetStatusAble pTray, Exception pException) {
      if (pTray == null) {
         pException.printStackTrace();
         return;
      }

      pTray.setError(pException);
   }

   public void exit() {
      System.exit(0);
   }

}
